"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const lockfile_file_1 = require("@pnpm/lockfile-file");
const R = require("ramda");
const table_1 = require("table");
const outdated_1 = require("../outdated");
const DEP_PRIORITY = {
    dependencies: 1,
    devDependencies: 2,
    optionalDependencies: 0,
};
exports.default = async (pkgs, args, cmd, opts) => {
    const outdatedByNameAndType = {};
    if (opts.lockfileDirectory) {
        const outdatedPackagesByProject = await outdated_1.outdatedDependenciesOfWorkspacePackages(pkgs, args, opts);
        for (let { prefix, outdatedPackages, manifest } of outdatedPackagesByProject) {
            outdatedPackages.forEach((outdatedPkg) => {
                const key = JSON.stringify([outdatedPkg.packageName, outdatedPkg.belongsTo]);
                if (!outdatedByNameAndType[key]) {
                    outdatedByNameAndType[key] = Object.assign(Object.assign({}, outdatedPkg), { dependentPkgs: [] });
                }
                outdatedByNameAndType[key].dependentPkgs.push({ location: prefix, manifest });
            });
        }
    }
    else {
        await Promise.all(pkgs.map(async ({ manifest, path }) => {
            const { outdatedPackages } = (await outdated_1.outdatedDependenciesOfWorkspacePackages([{ manifest, path }], args, Object.assign(Object.assign({}, opts), { lockfileDirectory: path })))[0];
            outdatedPackages.forEach((outdatedPkg) => {
                const key = JSON.stringify([outdatedPkg.packageName, outdatedPkg.belongsTo]);
                if (!outdatedByNameAndType[key]) {
                    outdatedByNameAndType[key] = Object.assign(Object.assign({}, outdatedPkg), { dependentPkgs: [] });
                }
                outdatedByNameAndType[key].dependentPkgs.push({ location: lockfile_file_1.getLockfileImporterId(opts.prefix, path), manifest });
            });
        }));
    }
    const columnNames = ['Package', 'Current', 'Latest', 'Dependents', 'Details'];
    const data = [
        columnNames,
        ...R.sortWith([
            outdated_1.sortBySemverChange,
            (o1, o2) => o1.packageName.localeCompare(o2.packageName),
            (o1, o2) => DEP_PRIORITY[o1.belongsTo] - DEP_PRIORITY[o2.belongsTo],
        ], (Object.values(outdatedByNameAndType).map(outdated_1.toOutdatedWithVersionDiff)))
            .map((outdatedPkg) => [
            outdated_1.renderPackageName(outdatedPkg),
            outdated_1.renderCurrent(outdatedPkg),
            outdated_1.renderLatest(outdatedPkg),
            outdatedPkg.dependentPkgs
                .map(({ manifest, location }) => manifest.name || location)
                .sort()
                .join(', '),
            outdated_1.renderDetails(outdatedPkg),
        ]),
    ];
    process.stdout.write(table_1.table(data, Object.assign(Object.assign({}, outdated_1.TABLE_OPTIONS), { columns: Object.assign(Object.assign({}, outdated_1.TABLE_OPTIONS.columns), { 
            // Dependents column:
            3: {
                width: outdated_1.getCellWidth(data, 3, 30),
                wrapWord: true,
            } }) })));
};
